import React from "react";
import { x2c, y2r } from "../lib/converters";
import { zoneToArea, among, zoneShape, areaToZone } from "../lib/structs";
import { choose, select, drag, write, setEditorRect, setContextMenuPosition, setAutofillDraggingTo, updateTable, } from "../store/actions";
import { DUMMY_IMG } from "../constants";
import { Context } from "../store";
import { FormulaError } from "../formula/evaluator";
import { Autofill, } from "../lib/autofill";
export const Cell = React.memo(({ y, x }) => {
    const rowId = y2r(y);
    const colId = x2c(x);
    const address = `${colId}${rowId}`;
    const { store, dispatch } = React.useContext(Context);
    const cellRef = React.useRef(document.createElement("td"));
    const { table, editingCell, choosing, selectingZone, headerTopSelecting, headerLeftSelecting, copyingZone, cutting, searchQuery, matchingCells, matchingCellIndex, editorRef, showAddress, autofillDraggingTo, } = store;
    const [before, setBefore] = React.useState("");
    const matchingCell = matchingCells[matchingCellIndex];
    const selectingArea = zoneToArea(selectingZone); // (top, left) -> (bottom, right)
    const copyingArea = zoneToArea(copyingZone); // (top, left) -> (bottom, right)
    const editing = editingCell === address;
    const pointed = choosing.y === y && choosing.x === x;
    const _setEditorRect = React.useCallback(() => {
        const rect = cellRef.current.getBoundingClientRect();
        dispatch(setEditorRect({
            y: rect.top,
            x: rect.left,
            height: rect.height,
            width: rect.width,
        }));
    }, []);
    React.useEffect(() => {
        if (pointed) {
            _setEditorRect();
        }
    }, [pointed]);
    const cell = table.getByPoint({ y, x });
    const writeCell = (value) => {
        if (before !== value) {
            dispatch(write(value));
        }
        setBefore("");
    };
    let matching = false;
    if (searchQuery && table.stringify({ y, x }).indexOf(searchQuery) !== -1) {
        matching = true;
    }
    let errorMessage = "";
    let rendered;
    try {
        rendered = table.render({ y, x }, writeCell);
    }
    catch (e) {
        if (e instanceof FormulaError) {
            errorMessage = e.message;
            rendered = e.code;
        }
        else {
            errorMessage = e.message;
            rendered = "#UNKNOWN";
            console.error(e);
        }
        // TODO: debug flag
    }
    return (React.createElement("td", { key: x, ref: cellRef, "data-x": x, "data-y": y, "data-address": address, className: `gs-cell ${among(copyingArea, { y, x }) ? "gs-copying" : ""} ${among(selectingArea, { y, x }) ? "gs-selected" : ""} ${pointed ? "gs-pointed" : ""} ${editing ? "gs-editing" : ""} ${matching ? "gs-matching" : ""} ${matchingCell === address ? "gs-searching" : ""}`, style: Object.assign(Object.assign({}, cell === null || cell === void 0 ? void 0 : cell.style), getCellStyle({ target: { y, x }, store, pointed, selectingArea, copyingArea, autofillDraggingTo })), onContextMenu: (e) => {
            e.preventDefault();
            dispatch(setContextMenuPosition({ y: e.clientY, x: e.clientX }));
            return false;
        }, onClick: (e) => {
            var _a;
            if (autofillDraggingTo) {
                return false;
            }
            dispatch(setContextMenuPosition({ y: -1, x: -1 }));
            if (e.shiftKey) {
                dispatch(drag({ y, x }));
            }
            else {
                dispatch(choose({ y, x }));
                dispatch(select({ startY: -1, startX: -1, endY: -1, endX: -1 }));
            }
            (_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.focus();
        }, onDoubleClick: (e) => {
            var _a;
            e.preventDefault();
            const dblclick = document.createEvent("MouseEvents");
            dblclick.initEvent("dblclick", true, true);
            (_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.dispatchEvent(dblclick);
            return false;
        }, draggable: true, onDragStart: (e) => {
            if (autofillDraggingTo) {
                return false;
            }
            e.dataTransfer.setDragImage(DUMMY_IMG, 0, 0);
            dispatch(choose({ y, x }));
            dispatch(select({ startY: y, startX: x, endY: y, endX: x }));
        }, onDragEnd: (e) => {
            if (autofillDraggingTo) {
                const autofill = new Autofill(store, autofillDraggingTo);
                dispatch(updateTable(autofill.applied));
                dispatch(select(areaToZone(autofill.wholeArea)));
                dispatch(setAutofillDraggingTo(null));
                return false;
            }
            const { height: h, width: w } = zoneShape(selectingZone);
            if (h + w === 0) {
                dispatch(select({ startY: -1, startX: -1, endY: -1, endX: -1 }));
            }
        }, onDragEnter: () => {
            if (autofillDraggingTo) {
                dispatch(setAutofillDraggingTo({ x, y }));
                return false;
            }
            if (headerTopSelecting) {
                dispatch(drag({ y: table.getNumRows(), x }));
                return false;
            }
            if (headerLeftSelecting) {
                dispatch(drag({ y, x: table.getNumCols() }));
                return false;
            }
            dispatch(drag({ y, x }));
            return false;
        } },
        React.createElement("div", { className: `gs-cell-rendered-wrapper-outer` },
            React.createElement("div", { className: 'gs-cell-rendered-wrapper-inner', style: {
                    justifyContent: (cell === null || cell === void 0 ? void 0 : cell.justifyContent) || "left",
                    alignItems: (cell === null || cell === void 0 ? void 0 : cell.alignItems) || "start",
                } },
                errorMessage && (React.createElement("div", { className: "formula-error-triangle", title: errorMessage })),
                showAddress && React.createElement("div", { className: "gs-cell-label" }, address),
                React.createElement("div", { className: "gs-cell-rendered" }, rendered)),
            ((pointed && selectingArea.bottom === -1) ||
                (selectingArea.bottom === y && selectingArea.right === x)) &&
                React.createElement("div", { className: "gs-autofill-drag", draggable: true, onDragStart: (e) => {
                        e.dataTransfer.setDragImage(DUMMY_IMG, 0, 0);
                        dispatch(setAutofillDraggingTo({ x, y }));
                        e.stopPropagation();
                        //e.preventDefault();
                        //return false;
                    } }))));
});
const BORDER_POINTED = "solid 2px #0077ff";
const BORDER_SELECTED = "solid 1px #0077ff";
const BORDER_CUTTING = "dotted 2px #0077ff";
const BORDER_COPYING = "dashed 2px #0077ff";
const getCellStyle = ({ target, pointed, selectingArea, copyingArea, store, autofillDraggingTo }) => {
    const style = {};
    if (autofillDraggingTo) {
        const autofill = new Autofill(store, autofillDraggingTo);
        Object.assign(style, autofill.getCellStyle(target));
    }
    const { cutting } = store;
    const { y, x } = target;
    if (pointed) {
        style.borderTop = BORDER_POINTED;
        style.borderBottom = BORDER_POINTED;
        style.borderLeft = BORDER_POINTED;
        style.borderRight = BORDER_POINTED;
    }
    else {
        // selecting style
        const { top, left, bottom, right } = selectingArea;
        if (top === y && left <= x && x <= right) {
            style.borderTop = BORDER_SELECTED;
        }
        if (bottom === y && left <= x && x <= right) {
            style.borderBottom = BORDER_SELECTED;
        }
        if (left === x && top <= y && y <= bottom) {
            style.borderLeft = BORDER_SELECTED;
        }
        if (right === x && top <= y && y <= bottom) {
            style.borderRight = BORDER_SELECTED;
        }
    }
    // copy or cut style
    {
        const { top, left, bottom, right } = copyingArea;
        const border = cutting ? BORDER_CUTTING : BORDER_COPYING;
        if (top === y && left <= x && x <= right) {
            style.borderTop = border;
        }
        if (bottom === y && left <= x && x <= right) {
            style.borderBottom = border;
        }
        if (left === x && top <= y && y <= bottom) {
            style.borderLeft = border;
        }
        if (right === x && top <= y && y <= bottom) {
            style.borderRight = border;
        }
    }
    return style;
};
//# sourceMappingURL=Cell.js.map