var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import React from "react";
import { clip } from "../lib/clipboard";
import { undo, redo, copy, cut, paste, setContextMenuPosition, updateTable, } from "../store/actions";
import { areaToZone, zoneShape, zoneToArea } from "../lib/structs";
import { Context } from "../store";
export const ContextMenu = () => {
    const { store, dispatch } = React.useContext(Context);
    const { table, choosing, selectingZone, headerTopSelecting, headerLeftSelecting, editorRef, contextMenuPosition, } = store;
    const { y, x } = choosing;
    let { top: selectingTop, left: selectingLeft, bottom: selectingBottom, right: selectingRight, } = zoneToArea(selectingZone);
    if (selectingTop === -1) {
        [selectingTop, selectingLeft, selectingBottom, selectingRight] = [
            y,
            x,
            y,
            x,
        ];
    }
    const [tableHeight, tableWidth] = [table.getNumRows(), table.getNumCols()];
    const { height, width } = zoneShape(Object.assign(Object.assign({}, selectingZone), { base: 1 }));
    const { y: top, x: left } = contextMenuPosition;
    if (top === -1) {
        return null;
    }
    const historyIndex = table.getHistoryIndex();
    return (React.createElement("div", { className: "gs-contextmenu-modal", onClick: (e) => {
            e.preventDefault();
            dispatch(setContextMenuPosition({ y: -1, x: -1 }));
            return false;
        } },
        React.createElement("div", { className: 'gs-contextmenu', style: {
                top: top,
                left: left,
            } },
            React.createElement("ul", null,
                React.createElement("li", { className: "enabled", onClick: () => {
                        const area = clip(store);
                        dispatch(cut(areaToZone(area)));
                    } },
                    React.createElement("div", { className: "gs-menu-name" }, "Cut"),
                    React.createElement("div", { className: "gs-menu-shortcut" },
                        React.createElement("span", { className: "gs-menu-underline" }, "X"))),
                React.createElement("li", { className: "enabled", onClick: () => {
                        const area = clip(store);
                        dispatch(copy(areaToZone(area)));
                    } },
                    React.createElement("div", { className: "gs-menu-name" }, "Copy"),
                    React.createElement("div", { className: "gs-menu-shortcut" },
                        React.createElement("span", { className: "gs-menu-underline" }, "C"))),
                React.createElement("li", { className: "enabled", onClick: () => __awaiter(void 0, void 0, void 0, function* () {
                        var _a;
                        const text = ((_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.value) || "";
                        dispatch(paste({ text }));
                    }) },
                    React.createElement("div", { className: "gs-menu-name" }, "Paste"),
                    React.createElement("div", { className: "gs-menu-shortcut" },
                        React.createElement("span", { className: "gs-menu-underline" }, "V"))),
                React.createElement("li", { className: "gs-menu-divider" }),
                !headerTopSelecting && (React.createElement("li", { className: table.maxNumRows !== -1 &&
                        tableHeight + height > table.maxNumRows
                        ? "disabled"
                        : "enabled", onClick: (e) => {
                        const newTable = table.addRows({
                            y: selectingTop,
                            numRows: height,
                            baseY: selectingTop,
                            reflection: {
                                selectingZone,
                                choosing,
                            },
                        });
                        dispatch(updateTable(newTable));
                    } },
                    React.createElement("div", { className: "gs-menu-name" },
                        "Insert ",
                        height,
                        " row",
                        height > 0 && "s",
                        " above"))),
                !headerTopSelecting && (React.createElement("li", { className: table.maxNumRows !== -1 &&
                        tableHeight + height > table.maxNumRows
                        ? "disabled"
                        : "enabled", onClick: (e) => {
                        selectingZone.startY += height;
                        selectingZone.endY += height;
                        choosing.y += height;
                        const newTable = table.addRows({
                            y: selectingBottom + 1,
                            numRows: height,
                            baseY: selectingBottom,
                            reflection: {
                                selectingZone,
                                choosing,
                            },
                        });
                        dispatch(updateTable(newTable));
                    } },
                    React.createElement("div", { className: "gs-menu-name" },
                        "Insert ",
                        height,
                        " row",
                        height > 0 && "s",
                        " below"))),
                !headerLeftSelecting && (React.createElement("li", { className: table.maxNumCols !== -1 && tableWidth + width > table.maxNumCols
                        ? "disabled"
                        : "enabled", onClick: (e) => {
                        const newTable = table.addCols({
                            x: selectingLeft,
                            numCols: width,
                            baseX: selectingLeft,
                            reflection: {
                                selectingZone,
                                choosing,
                            },
                        });
                        dispatch(updateTable(newTable));
                    } },
                    React.createElement("div", { className: "gs-menu-name" },
                        "Insert ",
                        width,
                        " column",
                        width > 0 && "s",
                        " left"))),
                !headerLeftSelecting && (React.createElement("li", { className: table.maxNumCols !== -1 && tableWidth + width > table.maxNumCols
                        ? "disabled"
                        : "enabled", onClick: (e) => {
                        selectingZone.startX += width;
                        selectingZone.endX += width;
                        choosing.x += width;
                        const newTable = table.addCols({
                            x: selectingRight + 1,
                            numCols: width,
                            baseX: selectingRight,
                            reflection: {
                                selectingZone,
                                choosing,
                            },
                        });
                        dispatch(updateTable(newTable));
                    } },
                    React.createElement("div", { className: "gs-menu-name" },
                        "Insert ",
                        width,
                        " column",
                        width > 0 && "s",
                        " right"))),
                !headerTopSelecting && (React.createElement("li", { className: table.minNumRows !== -1 &&
                        tableHeight - height < table.minNumRows
                        ? "disabled"
                        : "enabled", onClick: (e) => {
                        const newTable = table.removeRows({
                            y: selectingTop,
                            numRows: height,
                            reflection: {
                                selectingZone,
                                choosing,
                            },
                        });
                        dispatch(updateTable(newTable));
                    } },
                    React.createElement("div", { className: "gs-menu-name" },
                        "Remove ",
                        height,
                        " row",
                        height > 0 && "s"))),
                !headerLeftSelecting && (React.createElement("li", { className: table.minNumCols !== -1 && tableWidth - width < table.minNumCols
                        ? "disabled"
                        : "enabled", onClick: (e) => {
                        const newTable = table.removeCols({
                            x: selectingLeft,
                            numCols: width,
                            reflection: {
                                selectingZone,
                                choosing,
                            },
                        });
                        dispatch(updateTable(newTable));
                    } },
                    React.createElement("div", { className: "gs-menu-name" },
                        "Remove ",
                        width,
                        " column",
                        width > 0 && "s"))),
                (historyIndex > -1 || historyIndex < table.getHistorySize() - 1) && (React.createElement("li", { className: "gs-menu-divider" })),
                historyIndex > -1 && (React.createElement("li", { onClick: () => __awaiter(void 0, void 0, void 0, function* () {
                        dispatch(undo(null));
                    }) },
                    React.createElement("div", { className: "gs-menu-name" }, "Undo"),
                    React.createElement("div", { className: "gs-menu-shortcut" },
                        React.createElement("span", { className: "gs-menu-underline" }, "Z")))),
                historyIndex < table.getHistorySize() - 1 && (React.createElement("li", { onClick: () => __awaiter(void 0, void 0, void 0, function* () {
                        dispatch(redo(null));
                    }) },
                    React.createElement("div", { className: "gs-menu-name" }, "Redo"),
                    React.createElement("div", { className: "gs-menu-shortcut" },
                        React.createElement("span", { className: "gs-menu-underline" }, "R"))))))));
};
//# sourceMappingURL=ContextMenu.js.map