import * as React from "react";
import { DEFAULT_HEIGHT, DEFAULT_WIDTH, HEADER_HEIGHT, HEADER_WIDTH, SHEET_HEIGHT, SHEET_WIDTH, } from "../constants";
import { Context } from "../store";
import { reducer } from "../store/actions";
import { StoreInitializer } from "./StoreInitializer";
import { Resizer } from "./Resizer";
import { Emitter } from "./Emitter";
import { ContextMenu } from "./ContextMenu";
import { Table } from "../lib/table";
import { Tabular } from "./Tabular";
import { getMaxSizesFromCells } from "../lib/structs";
import { x2c, y2r } from "../lib/converters";
import { useEffect } from "react";
import { embedStyle } from "../styles/embedder";
export const GridSheet = ({ initial, tableRef, options = {}, className, style, additionalFunctions = {}, }) => {
    const { sheetResize: resize = "both" } = options;
    useEffect(() => {
        embedStyle();
    }, []);
    const sheetRef = React.useRef(document.createElement("div"));
    const searchInputRef = React.useRef(document.createElement("input"));
    const editorRef = React.useRef(document.createElement("textarea"));
    const gridOuterRef = React.useRef(document.createElement("div"));
    const initialState = {
        table: new Table({}),
        tableInitialized: false,
        sheetRef,
        searchInputRef,
        editorRef,
        gridOuterRef,
        choosing: { y: -1, x: -1 },
        cutting: false,
        selectingZone: { startY: -1, startX: -1, endY: -1, endX: -1 },
        copyingZone: { startY: -1, startX: -1, endY: -1, endX: -1 },
        autofillDraggingTo: null,
        headerTopSelecting: false,
        headerLeftSelecting: false,
        editingCell: "",
        editorRect: { y: 0, x: 0, height: 0, width: 0 },
        resizingRect: { y: -1, x: -1, height: -1, width: -1 },
        sheetHeight: 0,
        sheetWidth: 0,
        headerHeight: 0,
        headerWidth: 0,
        entering: false,
        matchingCells: [],
        matchingCellIndex: 0,
        editingOnEnter: true,
        showAddress: true,
        contextMenuPosition: { y: -1, x: -1 },
        resizingPositionY: [-1, -1, -1],
        resizingPositionX: [-1, -1, -1],
        minNumRows: 1,
        maxNumRows: -1,
        minNumCols: 1,
        maxNumCols: -1,
    };
    const [store, dispatch] = React.useReducer(reducer, initialState);
    const [sheetHeight, setSheetHeight] = React.useState(options.sheetHeight || estimateSheetHeight({ options, initial }));
    const [sheetWidth, setSheetWidth] = React.useState(options.sheetWidth || estimateSheetWidth({ options, initial }));
    React.useEffect(() => {
        const intervalId = window.setInterval(() => {
            var _a, _b, _c, _d;
            if ((_a = sheetRef.current) === null || _a === void 0 ? void 0 : _a.clientHeight) {
                setSheetHeight((_b = sheetRef.current) === null || _b === void 0 ? void 0 : _b.clientHeight);
            }
            if ((_c = sheetRef.current) === null || _c === void 0 ? void 0 : _c.clientWidth) {
                setSheetWidth((_d = sheetRef.current) === null || _d === void 0 ? void 0 : _d.clientWidth);
            }
        }, 1000);
        return () => window.clearInterval(intervalId);
    }, []);
    const { onChange, onSelect, mode } = options;
    return (React.createElement(Context.Provider, { value: { store, dispatch } },
        React.createElement("div", { ref: sheetRef, className: `gridsheet-1 ${mode || "light"} ${className || ""}`, style: Object.assign(Object.assign({}, style), { resize, height: sheetHeight, width: sheetWidth, maxWidth: store.table.totalWidth + 3, maxHeight: store.table.totalHeight + 3 }) },
            React.createElement(Tabular, { tableRef: tableRef }),
            React.createElement(StoreInitializer, { initial: initial, options: Object.assign(Object.assign({}, options), { sheetHeight, sheetWidth }), additionalFunctions: additionalFunctions }),
            React.createElement(ContextMenu, null),
            React.createElement(Resizer, null),
            React.createElement(Emitter, { onChange: onChange, onSelect: onSelect }))));
};
const estimateSheetHeight = ({ initial, options }) => {
    var _a, _b;
    const auto = getMaxSizesFromCells(initial);
    let estimatedHeight = options.headerHeight || HEADER_HEIGHT;
    for (let y = 0; y < auto.numRows; y++) {
        const row = y2r(y);
        const height = ((_a = initial === null || initial === void 0 ? void 0 : initial[row]) === null || _a === void 0 ? void 0 : _a.height) || ((_b = initial === null || initial === void 0 ? void 0 : initial.default) === null || _b === void 0 ? void 0 : _b.height) || DEFAULT_HEIGHT;
        if (estimatedHeight + height > SHEET_HEIGHT) {
            return SHEET_HEIGHT;
        }
        estimatedHeight += height;
    }
    return estimatedHeight + 3;
};
const estimateSheetWidth = ({ initial, options }) => {
    var _a, _b;
    const auto = getMaxSizesFromCells(initial);
    let estimatedWidth = options.headerWidth || HEADER_WIDTH;
    for (let x = 0; x < auto.numCols; x++) {
        const col = x2c(x);
        const width = ((_a = initial === null || initial === void 0 ? void 0 : initial[col]) === null || _a === void 0 ? void 0 : _a.width) || ((_b = initial === null || initial === void 0 ? void 0 : initial.default) === null || _b === void 0 ? void 0 : _b.width) || DEFAULT_WIDTH;
        if (estimatedWidth + width > SHEET_WIDTH) {
            return SHEET_WIDTH;
        }
        estimatedWidth += width;
    }
    return estimatedWidth + 3;
};
//# sourceMappingURL=GridSheet.js.map