import React from "react";
import { Context } from "../store";
import { setResizingPositionY, setResizingPositionX, updateTable, } from "../store/actions";
import { DEFAULT_HEIGHT, DEFAULT_WIDTH, MIN_WIDTH, MIN_HEIGHT, } from "../constants";
import { zoneToArea, makeSequence, between } from "../lib/structs";
import { p2a } from "../lib/converters";
export const Resizer = React.memo(() => {
    const { store, dispatch } = React.useContext(Context);
    const { resizingPositionY: posY, resizingPositionX: posX, table, headerTopSelecting, headerLeftSelecting, selectingZone, editorRef, sheetRef, } = store;
    const [y, startY, endY] = posY;
    const [x, startX, endX] = posX;
    if (y === -1 && x === -1) {
        return null;
    }
    const cell = table.getByPoint({ y: y === -1 ? 0 : y, x: x === -1 ? 0 : x });
    const { y: offsetY, x: offsetX } = sheetRef.current.getBoundingClientRect();
    const baseWidth = (cell === null || cell === void 0 ? void 0 : cell.width) || DEFAULT_WIDTH;
    const baseHeight = (cell === null || cell === void 0 ? void 0 : cell.height) || DEFAULT_HEIGHT;
    const width = baseWidth + (endX - startX);
    const height = baseHeight + (endY - startY);
    const handleResizeEnd = () => {
        var _a;
        const selectingArea = zoneToArea(selectingZone);
        const { top, left, bottom, right } = selectingArea;
        const diff = {};
        if (x !== -1) {
            let xs = [x];
            if (headerTopSelecting &&
                between({ start: left, end: right }, x)) {
                xs = makeSequence(left, right + 1);
            }
            xs.forEach((x, i) => {
                diff[p2a({ y: 0, x })] = { width };
            });
        }
        if (y !== -1) {
            let ys = [y];
            if (headerLeftSelecting && between({ start: top, end: bottom }, y)) {
                ys = makeSequence(top, bottom + 1);
            }
            ys.forEach((y, i) => {
                diff[p2a({ y, x: 0 })] = { height };
            });
        }
        const newTable = table.update({
            diff,
            partial: true,
            reflection: { selectingZone },
        });
        dispatch(updateTable(newTable));
        dispatch(setResizingPositionY([-1, -1, -1]));
        dispatch(setResizingPositionX([-1, -1, -1]));
        (_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    };
    const handleResizeMove = (e) => {
        if (y !== -1) {
            let endY = e.clientY;
            const height = baseHeight + (endY - startY);
            if (height < MIN_HEIGHT) {
                endY += MIN_HEIGHT - height;
            }
            dispatch(setResizingPositionY([y, startY, endY]));
        }
        else if (x !== -1) {
            let endX = e.clientX;
            const width = baseWidth + (endX - startX);
            if (width < MIN_WIDTH) {
                endX += MIN_WIDTH - width;
            }
            dispatch(setResizingPositionX([x, startX, endX]));
        }
    };
    return (React.createElement("div", { className: "gs-resizing", onMouseUp: handleResizeEnd, onMouseMove: handleResizeMove },
        x !== -1 && (React.createElement("div", { className: 'gs-line', style: { width: 1, height: "100%", left: endX - offsetX } },
            React.createElement("span", { style: { left: "-50%" } },
                width,
                "px"))),
        y !== -1 && (React.createElement("div", { className: 'gs-line', style: { width: "100%", height: 1, top: endY - offsetY } },
            React.createElement("span", { style: { top: "-50%" } },
                height,
                "px")))));
});
//# sourceMappingURL=Resizer.js.map