import { a2p } from "../lib/converters";
import React from "react";
import { Context } from "../store";
import { setSearchQuery, search } from "../store/actions";
import { smartScroll } from "../lib/virtualization";
export const SearchBox = () => {
    const { store, dispatch } = React.useContext(Context);
    const { editorRef, searchInputRef, gridOuterRef, searchQuery, matchingCellIndex, matchingCells, table, } = store;
    const matchingCell = matchingCells[matchingCellIndex];
    React.useEffect(() => {
        if (!matchingCell) {
            return;
        }
        const point = a2p(matchingCell);
        if (typeof point === "undefined") {
            return;
        }
        smartScroll(table, gridOuterRef.current, point);
    }, [searchQuery, matchingCellIndex]);
    if (typeof searchQuery === "undefined") {
        return null;
    }
    return (React.createElement("div", { className: "gs-search" },
        React.createElement("div", { className: "gs-searchbox", title: "Press 'Enter' to next, 'Enter + Shift' to previous." },
            React.createElement("input", { type: "text", ref: searchInputRef, value: searchQuery, placeholder: "Search", autoFocus: true, onChange: (e) => dispatch(setSearchQuery(e.target.value)), onKeyDown: (e) => {
                    var _a;
                    if (e.key === "Escape") {
                        (_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.focus();
                        dispatch(setSearchQuery(undefined));
                    }
                    if (e.key === "f" && (e.ctrlKey || e.metaKey)) {
                        e.preventDefault();
                        return false;
                    }
                    if (e.key !== "Enter") {
                        return true;
                    }
                    dispatch(search(e.shiftKey ? -1 : 1));
                    return false;
                } }),
            React.createElement("div", { className: "gs-search-progress", onClick: (e) => {
                    const input = e.currentTarget.previousSibling;
                    (input === null || input === void 0 ? void 0 : input.nodeName) === "INPUT" && input.focus();
                } },
                matchingCells.length === 0 ? 0 : matchingCellIndex + 1,
                " /",
                " ",
                matchingCells.length)),
        React.createElement("div", { className: "gs-search-close" },
            React.createElement("a", { onClick: () => dispatch(setSearchQuery(undefined)) }, "Close"))));
};
//# sourceMappingURL=SearchBox.js.map