import React from "react";
import { Editor } from "./Editor";
import { Cell } from "./Cell";
import { HeaderTopCell } from "./HeaderTopCell";
import { HeaderLeftCell } from "./HeaderLeftCell";
import { SearchBox } from "./SearchBox";
import { Context } from "../store";
import { choose, select, setEntering, updateTable } from "../store/actions";
import { virtualize } from "../lib/virtualization";
export const createTableRef = () => React.useRef(null);
export const Tabular = ({ tableRef }) => {
    var _a, _b;
    const { store, dispatch } = React.useContext(Context);
    const { sheetHeight, sheetWidth, headerHeight, headerWidth, table, tableInitialized, gridOuterRef, sheetRef, } = store;
    React.useEffect(() => {
        if (tableRef && tableInitialized) {
            tableRef.current = {
                table,
                dispatch: (table) => {
                    dispatch(updateTable(table));
                },
            };
        }
    }, [table]);
    const [virtualized, setVirtualized] = React.useState(virtualize(table, gridOuterRef.current));
    React.useEffect(() => {
        setVirtualized(virtualize(table, gridOuterRef.current));
    }, [
        gridOuterRef.current,
        table,
        (_a = sheetRef.current) === null || _a === void 0 ? void 0 : _a.clientHeight,
        (_b = sheetRef.current) === null || _b === void 0 ? void 0 : _b.clientWidth,
    ]);
    const { ys, xs, adjuster } = virtualized;
    return (React.createElement(React.Fragment, null,
        React.createElement(Editor, null),
        React.createElement(SearchBox, null),
        React.createElement("div", { className: "gs-tabular", style: {
                width: sheetWidth,
                height: sheetHeight,
            }, ref: gridOuterRef, onMouseEnter: () => {
                dispatch(setEntering(true));
            }, onMouseLeave: () => {
                dispatch(setEntering(false));
            }, onScroll: (e) => {
                setVirtualized(virtualize(table, e.currentTarget));
            } },
            React.createElement("div", { className: "gs-tabular-inner", style: {
                    width: table.totalWidth,
                    height: table.totalHeight,
                } },
                React.createElement("table", { style: {
                        width: table.totalWidth,
                    } },
                    React.createElement("thead", null,
                        React.createElement("tr", { style: { height: headerHeight } },
                            React.createElement("th", { className: "gs-header gs-header-left gs-header-top gs-header-left-top", style: { width: headerWidth, position: "sticky" }, onClick: () => {
                                    dispatch(choose({ y: -1, x: -1 }));
                                    window.setTimeout(() => {
                                        dispatch(choose({ y: 1, x: 1 }));
                                        dispatch(select({
                                            startY: 1,
                                            startX: 1,
                                            endY: table.getNumRows(),
                                            endX: table.getNumCols(),
                                        }));
                                    }, 100);
                                } }),
                            React.createElement("th", { className: "gs-adjuster", style: { width: adjuster.left } }),
                            xs.map((x) => React.createElement(HeaderTopCell, { x: x, key: x })),
                            React.createElement("th", { className: "gs-adjuster", style: { width: adjuster.right } }))),
                    React.createElement("tbody", null,
                        React.createElement("tr", null,
                            React.createElement("th", { className: "gs-header gs-header-left gs-adjuster", style: { height: adjuster.top } }),
                            xs.map((x) => React.createElement("td", { className: "gs-adjuster", key: x })))),
                    React.createElement("tbody", null, ys.map((y, i) => {
                        return (React.createElement("tr", { key: y },
                            React.createElement(HeaderLeftCell, { y: y }),
                            React.createElement("td", { className: "gs-adjuster" }),
                            xs.map((x) => React.createElement(Cell, { key: x, y: y, x: x })),
                            React.createElement("td", { className: "gs-adjuster" })));
                    })))))));
};
//# sourceMappingURL=Tabular.js.map