import { FormulaError } from "../evaluator";
import { BaseFunction } from "./__base";
import { ensureNumber } from "./__utils";
export class CosFunction extends BaseFunction {
    constructor() {
        super(...arguments);
        this.example = "COS(PI()/2)";
        this.helpText = ["Returns the cos of the angle specified in radians."];
        this.helpArgs = [
            {
                name: "angle",
                description: "An angle in radians, at which you want the cos.",
            },
        ];
    }
    validate() {
        if (this.bareArgs.length !== 1) {
            throw new FormulaError("#N/A", "Number of arguments for COS is incorrect.");
        }
        this.bareArgs = this.bareArgs.map((arg) => ensureNumber(arg));
    }
    main(angle) {
        return Math.cos(angle);
    }
}
//# sourceMappingURL=cos.js.map