import { solveTable } from "../solver";
import { Table } from "../../lib/table";
import { BaseFunction } from "./__base";
import { ensureNumber } from "./__utils";
export class CountaFunction extends BaseFunction {
    constructor() {
        super(...arguments);
        this.example = "COUNTA(A2:A100,B2:B100,4,26)";
        this.helpText = ["Returns the number of values in the data set."];
        this.helpArgs = [
            { name: "value1", description: "First number or range." },
            {
                name: "value2",
                description: "Additional numbers or ranges",
                optional: true,
                iterable: true,
            },
        ];
    }
    validate() {
        const spreaded = [];
        this.bareArgs.map((arg) => {
            if (arg instanceof Table) {
                spreaded.push(...solveTable({ table: arg }).reduce((a, b) => a.concat(b)));
                return;
            }
            spreaded.push(ensureNumber(arg));
        });
        this.bareArgs = spreaded;
    }
    main(...values) {
        return values.filter((v) => v != null && v !== "").length;
    }
}
//# sourceMappingURL=counta.js.map