import { FormulaError } from "../evaluator";
import { BaseFunction } from "./__base";
import { ensureNumber } from "./__utils";
export class LtFunction extends BaseFunction {
    constructor() {
        super(...arguments);
        this.example = "LT(3, 6)";
        this.helpText = [
            "Returns TRUE if the first argument is truely less than the second argument, FALSE otherwise.",
            "This is the same as the '<' operator.",
        ];
        this.helpArgs = [
            { name: "value1", description: "First value." },
            { name: "value2", description: "A value to be compared with value1." },
        ];
    }
    validate() {
        if (this.bareArgs.length !== 2) {
            throw new FormulaError("#N/A", "Number of arguments for LT is incorrect.");
        }
        this.bareArgs = this.bareArgs.map((arg) => ensureNumber(arg));
    }
    main(v1, v2) {
        return v1 < v2;
    }
}
//# sourceMappingURL=lt.js.map