import { FormulaError } from "../evaluator";
import { BaseFunction } from "./__base";
import { ensureNumber } from "./__utils";
export class PowerFunction extends BaseFunction {
    constructor() {
        super(...arguments);
        this.example = "POWER(4,0.5)";
        this.helpText = ["Returns a number multiplied by an exponent."];
        this.helpArgs = [
            { name: "base", description: "A number to be multiplied by an exponent." },
            { name: "exponent", description: "An exponent to power the base." },
        ];
    }
    validate() {
        if (this.bareArgs.length !== 2) {
            throw new FormulaError("#N/A", "Number of arguments for POWER is incorrect.");
        }
        this.bareArgs = this.bareArgs.map((arg) => ensureNumber(arg));
    }
    main(base, exponent) {
        return Math.pow(base, exponent);
    }
}
//# sourceMappingURL=power.js.map