import { solveTable } from "../solver";
import { Table } from "../../lib/table";
import { BaseFunction } from "./__base";
import { ensureNumber } from "./__utils";
export class ProductFunction extends BaseFunction {
    constructor() {
        super(...arguments);
        this.example = "PRODUCT(A2:A100)";
        this.helpText = ["Returns the product of a series of numbers or cells."];
        this.helpArgs = [
            { name: "value1", description: "First number or range." },
            {
                name: "value2",
                description: "Additional numbers or ranges",
                optional: true,
                iterable: true,
            },
        ];
    }
    validate() {
        const spreaded = [];
        this.bareArgs.forEach((arg) => {
            if (arg instanceof Table) {
                spreaded.push(...solveTable({ table: arg })
                    .reduce((a, b) => a.concat(b))
                    .filter((v) => typeof v === "number"));
                return;
            }
            spreaded.push(ensureNumber(arg));
        });
        this.bareArgs = spreaded;
    }
    main(...values) {
        return values.reduce((a, b) => a * b);
    }
}
//# sourceMappingURL=product.js.map