import { FormulaError } from "../evaluator";
import { BaseFunction } from "./__base";
import { ensureNumber } from "./__utils";
export class RadiansFunction extends BaseFunction {
    constructor() {
        super(...arguments);
        this.example = "RADIANS(180)";
        this.helpText = ["Converts an angle from degrees to radians."];
        this.helpArgs = [
            {
                name: "angle",
                description: "The angle to convert from degrees to radians.",
            },
        ];
    }
    validate() {
        if (this.bareArgs.length !== 1) {
            throw new FormulaError("#N/A", "Number of arguments for RADIANS is incorrect.");
        }
        this.bareArgs = this.bareArgs.map((arg) => ensureNumber(arg));
    }
    main(angle) {
        return (angle / 180) * Math.PI;
    }
}
//# sourceMappingURL=radians.js.map