import { solveTable } from "../solver";
import { Table } from "../../lib/table";
import { BaseFunction } from "./__base";
import { ensureNumber } from "./__utils";
import { FormulaError } from "../evaluator";
export class SumFunction extends BaseFunction {
    constructor() {
        super(...arguments);
        this.example = "SUM(A2:A100, 101)";
        this.helpText = ["Returns the sum of a series of numbers or cells."];
        this.helpArgs = [
            { name: "value1", description: "First number or range." },
            {
                name: "value2",
                description: "Additional numbers or ranges",
                optional: true,
                iterable: true,
            },
        ];
    }
    validate() {
        if (this.bareArgs.length === 0) {
            throw new FormulaError("#N/A", "One or more arguments are required.");
        }
        const spreaded = [];
        this.bareArgs.forEach((arg) => {
            if (arg instanceof Table) {
                spreaded.push(...solveTable({ table: arg })
                    .reduce((a, b) => a.concat(b))
                    .filter((v) => typeof v === "number"));
                return;
            }
            spreaded.push(ensureNumber(arg));
        });
        this.bareArgs = spreaded;
    }
    main(...values) {
        if (values.length === 0) {
            return 0;
        }
        return values.reduce((a, b) => a + b);
    }
}
//# sourceMappingURL=sum.js.map