import { FormulaError } from "../evaluator";
import { solveTable } from "../solver";
import { Table } from "../../lib/table";
import { BaseFunction } from "./__base";
import { check } from "./__utils";
export class SumifFunction extends BaseFunction {
    constructor() {
        super(...arguments);
        this.example = 'SUMIF(A1:A10,">20")';
        this.helpText = ["Returns the sum of a series of cells."];
        this.helpArgs = [
            { name: "range1", description: "A condition range." },
            {
                name: "condition",
                description: "A condition for summarization.",
            },
            {
                name: "range2",
                description: "A range to be summarized.",
                optional: true,
            },
        ];
    }
    validate() {
        if (this.bareArgs.length !== 2 && this.bareArgs.length !== 3) {
            throw new FormulaError("#N/A", "Number of arguments for SUMIF is incorrect.");
        }
        if (this.bareArgs[2] != undefined && this.bareArgs[2] instanceof Table) {
            throw new FormulaError("#N/A", "3rd argument must be range.");
        }
    }
    main(range, condition, sumRange) {
        if (!(range instanceof Table)) {
            return check(range, condition) ? range : 0;
        }
        const conditionMatrix = solveTable({ table: range });
        let sumMatrix = conditionMatrix;
        if (sumRange) {
            const [top, left] = [sumRange.top, sumRange.left];
            const area = {
                top,
                left,
                bottom: top + sumRange.getNumRows(),
                right: left + sumRange.getNumCols(),
            };
            sumMatrix = solveTable({ table: this.table.trim(area) });
        }
        let total = 0;
        conditionMatrix.forEach((row, y) => row.forEach((c, x) => {
            var _a;
            const s = ((_a = sumMatrix[y]) === null || _a === void 0 ? void 0 : _a[x]) || 0;
            if (typeof s === "number" && check(c, condition)) {
                total += s;
            }
        }));
        return total;
    }
}
//# sourceMappingURL=sumif.js.map