var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { a2p, p2a } from "./converters";
export const slideArea = (area, y, x) => {
    const { top, left, bottom, right } = area;
    return { top: top + y, left: left + x, bottom: bottom + y, right: right + x };
};
export const superposeArea = (srcArea, dstArea) => {
    const { height: srcHeight, width: srcWidth } = areaShape(srcArea);
    const { height: dstHeight, width: dstWidth } = areaShape(dstArea);
    // biggerHeight, biggerWidth
    return {
        height: srcHeight > dstHeight ? srcHeight : dstHeight,
        width: srcWidth > dstWidth ? srcWidth : dstWidth,
    };
};
export const concatAreas = (area1, area2) => {
    const result = Object.assign({}, area1);
    if (area2.left < area1.left) {
        result.left = area2.left;
    }
    if (area2.right > area1.right) {
        result.right = area2.right;
    }
    if (area2.top < area1.top) {
        result.top = area2.top;
    }
    if (area2.bottom > area1.bottom) {
        result.bottom = area2.bottom;
    }
    return result;
};
export const zoneToArea = (zone) => {
    const [top, bottom] = zone.startY < zone.endY
        ? [zone.startY, zone.endY]
        : [zone.endY, zone.startY];
    const [left, right] = zone.startX < zone.endX
        ? [zone.startX, zone.endX]
        : [zone.endX, zone.startX];
    return { top, left, bottom, right };
};
export const areaToZone = (area) => {
    return {
        startY: area.top,
        startX: area.left,
        endY: area.bottom,
        endX: area.right,
    };
};
export const areaToRange = (area) => {
    const { top, left, bottom, right } = area;
    return `${p2a({ y: top, x: left })}${p2a({
        y: bottom,
        x: right,
    })}`;
};
export const rangeToArea = (range) => {
    const cells = range.split(":");
    const [start, end] = cells;
    const { y: top, x: left } = a2p(start);
    const { y: bottom, x: right } = a2p(end);
    return { top, left, bottom, right };
};
export const between = (range, index) => {
    if (range.start === -1 || range.end === -1) {
        return false;
    }
    return ((range.start <= index && index <= range.end) ||
        (range.end <= index && index <= range.start));
};
export const among = (area, point) => {
    if (area.top === -1 ||
        area.left === -1 ||
        area.bottom === -1 ||
        area.right === -1) {
        return false;
    }
    const { y, x } = point;
    const { top, left, bottom, right } = area;
    return top <= y && y <= bottom && left <= x && x <= right;
};
export const zoneShape = (_a) => {
    var { base = 0 } = _a, zone = __rest(_a, ["base"]);
    return {
        height: base + Math.abs(zone.startY - zone.endY),
        width: base + Math.abs(zone.startX - zone.endX),
    };
};
export const areaShape = (_a) => {
    var { base = 0 } = _a, area = __rest(_a, ["base"]);
    return {
        height: base + Math.abs(area.top - area.bottom),
        width: base + Math.abs(area.left - area.right),
    };
};
export const matrixShape = ({ base = 0, matrix, }) => {
    const h = matrix.length;
    if (h === 0) {
        return { height: 0, width: 0 };
    }
    return { height: base + h, width: base + matrix[0].length };
};
export const makeSequence = (start, stop, step = 1) => {
    return Array.from({ length: (stop - start - 1) / step + 1 }, (_, i) => start + i * step);
};
export const oa2aa = (oa, fields) => {
    const aa = [];
    oa.forEach((o) => {
        const a = [];
        fields.forEach((field) => {
            a.push(o[field]);
        });
        aa.push(a);
    });
    return aa;
};
export const aa2oa = (aa, fields) => {
    const oa = [];
    aa.forEach((a) => {
        const o = {};
        a.forEach((v, i) => {
            if (i >= fields.length) {
                return;
            }
            const field = fields[i];
            o[field] = v;
        });
        oa.push(o);
    });
    return oa;
};
export const putMatrix = (dst, src, dstArea) => {
    const lostRows = {};
    const { top, left, bottom, right } = dstArea;
    const { height: dstNumRows, width: dstNumCols } = matrixShape({
        matrix: dst,
        base: 1,
    });
    for (let y = top; y <= bottom; y++) {
        const lostRow = [];
        for (let x = left; x <= right; x++) {
            const value = src[y - top][x - left];
            // -1 means excluding headers
            if (y < dstNumRows - 1 && x < dstNumCols - 1) {
                dst[y][x] = value;
                continue;
            }
            if (lostRow.length === 0) {
                lostRows[p2a({ y, x })] = [lostRow];
            }
            lostRow.push(value);
        }
    }
    return lostRows;
};
export const createMatrix = (numRows, numCols, fill) => {
    return [...Array(numRows)].map(() => Array(numCols).fill(fill));
};
export const cropMatrix = (matrix, area) => {
    const { top, left, bottom, right } = area;
    return matrix
        .slice(top, bottom + 1)
        .map((cols) => cols.slice(left, right + 1));
};
export const generateInitialSimple = ({ cells = {}, ensured = {}, matrix = [], flattenAs = "value", }) => {
    return generateInitial({
        cells,
        ensured,
        matrices: { A1: matrix },
        flattenAs,
    });
};
export const generateInitial = ({ cells = {}, ensured = {}, matrices = {}, flattenAs = "value", } = {}) => {
    upsert({ cells, flattenAs, matrices });
    const { numRows, numCols } = Object.assign({ numRows: 1, numCols: 1 }, ensured);
    const rightBottom = p2a({ y: numRows, x: numCols });
    if (cells[rightBottom] == null) {
        cells[rightBottom] = {};
    }
    return cells;
};
export const upsert = ({ cells = {}, matrices = {}, flattenAs, }) => {
    Object.keys(matrices).forEach((address) => {
        const matrix = matrices[address];
        const { y: baseY, x: baseX } = a2p(address);
        matrix.forEach((row, y) => {
            row.forEach((e, x) => {
                const id = p2a({ y: baseY + y, x: baseX + x });
                if (flattenAs) {
                    const cell = cells[id];
                    cells[id] = Object.assign({ [flattenAs]: e }, cell);
                }
                else {
                    cells[id] = e;
                }
            });
        });
    });
    return cells;
};
export const getMaxSizesFromCells = (cells = {}) => {
    let [lastY, lastX] = [0, 0];
    Object.keys(cells).map((address) => {
        const { y, x } = a2p(address);
        if (lastY < y) {
            lastY = y;
        }
        if (lastX < x) {
            lastX = x;
        }
    });
    return { numRows: lastY, numCols: lastX };
};
export const range = (start, end) => {
    const list = [];
    for (let i = start; i <= end; i++) {
        list.push(i);
    }
    return list;
};
export const complementSelectingArea = (selectingArea, choosing) => {
    if (selectingArea.left === -1) {
        selectingArea = { left: choosing.x, top: choosing.y, right: choosing.x, bottom: choosing.y };
    }
    return selectingArea;
};
export const isSameArea = (area1, area2) => {
    if (area1.top !== area2.top) {
        return false;
    }
    if (area1.left !== area2.left) {
        return false;
    }
    if (area1.bottom !== area2.bottom) {
        return false;
    }
    if (area1.right !== area2.right) {
        return false;
    }
    return true;
};
//# sourceMappingURL=structs.js.map