import { zoneToArea, superposeArea, matrixShape, areaShape, areaToZone, } from "../lib/structs";
import { tsv2matrix, p2a } from "../lib/converters";
import { DEFAULT_HEIGHT, DEFAULT_WIDTH } from "../constants";
import { initSearchStatement, restrictPoints } from "./helpers";
import { smartScroll } from "../lib/virtualization";
const actions = {};
export const reducer = (store, action) => {
    const act = actions[action.type];
    if (act == null) {
        return store;
    }
    return Object.assign(Object.assign({}, store), act.reduce(store, action.value));
};
export class CoreAction {
    constructor() {
        this.actionId = 1;
    }
    reduce(store, payload) {
        return store;
    }
    call(payload) {
        return {
            type: this.actionId,
            value: payload,
        };
    }
    bind() {
        this.actionId = CoreAction.head++;
        actions[this.actionId] = this;
        return this.call.bind(this);
    }
}
CoreAction.head = 1;
class SetSearchQueryAction extends CoreAction {
    reduce(store, payload) {
        const searchQuery = payload;
        const { table } = store;
        return Object.assign(Object.assign(Object.assign({}, store), initSearchStatement(table, Object.assign(Object.assign({}, store), { searchQuery }))), { searchQuery });
    }
}
export const setSearchQuery = new SetSearchQueryAction().bind();
class SetEditingCellAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { editingCell: payload });
    }
}
export const setEditingCell = new SetEditingCellAction().bind();
class SetEditingOnEnterAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { editingOnEnter: payload });
    }
}
export const setEditingOnEnter = new SetEditingOnEnterAction().bind();
class SetAutofillDraggingToAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { autofillDraggingTo: payload });
    }
}
export const setAutofillDraggingTo = new SetAutofillDraggingToAction().bind();
class SetShowAddressAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { showAddress: payload });
    }
}
export const setShowAddress = new SetShowAddressAction().bind();
class SetContextMenuPositionAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { contextMenuPosition: payload });
    }
}
export const setContextMenuPosition = new SetContextMenuPositionAction().bind();
class SetResizingPositionYAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { resizingPositionY: payload });
    }
}
export const setResizingPositionY = new SetResizingPositionYAction().bind();
class SetResizingPositionXAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { resizingPositionX: payload });
    }
}
export const setResizingPositionX = new SetResizingPositionXAction().bind();
class SetOnSaveAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { onSave: payload });
    }
}
export const setOnSave = new SetOnSaveAction().bind();
class SetEnteringAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { entering: payload });
    }
}
export const setEntering = new SetEnteringAction().bind();
class SetHeaderHeightAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { headerHeight: payload });
    }
}
export const setHeaderHeight = new SetHeaderHeightAction().bind();
class SetHeaderWidthAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { headerWidth: payload });
    }
}
export const setHeaderWidth = new SetHeaderWidthAction().bind();
class SetSheetHeightAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { sheetHeight: payload });
    }
}
export const setSheetHeight = new SetSheetHeightAction().bind();
class SetSheetWidthAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { sheetWidth: payload });
    }
}
export const setSheetWidth = new SetSheetWidthAction().bind();
class InitializeTableAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { table: payload, tableInitialized: true });
    }
}
export const initializeTable = new InitializeTableAction().bind();
class UpdateTableAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign(Object.assign(Object.assign({}, store), { table: payload }), initSearchStatement(payload, store)), restrictPoints(store, payload));
    }
}
export const updateTable = new UpdateTableAction().bind();
class SetEditorRectAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { editorRect: payload });
    }
}
export const setEditorRect = new SetEditorRectAction().bind();
class SetResizingRectAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { resizingRect: payload });
    }
}
export const setResizingRect = new SetResizingRectAction().bind();
class BlurAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { editingCell: "" });
    }
}
export const blur = new BlurAction().bind();
class CopyAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { copyingZone: payload, cutting: false });
    }
}
export const copy = new CopyAction().bind();
class CutAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { copyingZone: payload, cutting: true });
    }
}
export const cut = new CutAction().bind();
class PasteAction extends CoreAction {
    reduce(store, payload) {
        const { choosing, copyingZone, selectingZone, cutting, table } = store;
        let selectingArea = zoneToArea(selectingZone);
        const copyingArea = zoneToArea(copyingZone);
        if (cutting) {
            const src = copyingArea;
            const { height: h, width: w } = areaShape(copyingArea);
            const dst = selectingArea.top !== -1
                ? {
                    top: selectingArea.top,
                    left: selectingArea.left,
                    bottom: selectingArea.top + h,
                    right: selectingArea.left + w,
                }
                : {
                    top: choosing.y,
                    left: choosing.x,
                    bottom: choosing.y + h,
                    right: choosing.x + w,
                };
            const newTable = table.move({
                src,
                dst,
                reflection: {
                    selectingZone: areaToZone(dst),
                    copyingZone,
                    cutting,
                },
            });
            return Object.assign(Object.assign(Object.assign({}, store), initSearchStatement(newTable, store)), { cutting: false, table: newTable, selectingZone: areaToZone(dst), copyingZone: { startY: -1, startX: -1, endY: -1, endX: -1 } });
        }
        let newTable;
        let { y, x } = choosing;
        const { text } = payload;
        if (copyingArea.top === -1) {
            const matrixFrom = tsv2matrix(text);
            let { height, width } = matrixShape({ matrix: matrixFrom, base: -1 });
            selectingArea = {
                top: y,
                left: x,
                bottom: y + height,
                right: x + width,
            };
            newTable = table.writeMatrix({
                point: { y, x },
                matrix: matrixFrom,
                reflection: {
                    selectingZone: areaToZone(selectingArea),
                },
            });
        }
        else {
            let { height, width } = areaShape(copyingArea);
            if (selectingArea.top !== -1) {
                y = selectingArea.top;
                x = selectingArea.left;
                const superposed = superposeArea(selectingArea, copyingArea);
                height = superposed.height;
                width = superposed.width;
            }
            selectingArea = { top: y, left: x, bottom: y + height, right: x + width };
            newTable = table.copy({
                src: copyingArea,
                dst: selectingArea,
                reflection: {
                    copyingZone,
                    selectingZone,
                },
            });
        }
        return Object.assign(Object.assign(Object.assign({}, store), { table: newTable, selectingZone: areaToZone(selectingArea), copyingZone: { startY: -1, startX: -1, endY: -1, endX: -1 } }), initSearchStatement(newTable, store));
    }
}
export const paste = new PasteAction().bind();
class EscapeAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { copyingZone: { startY: -1, startX: -1, endY: -1, endX: -1 }, cutting: false, editingCell: "", headerTopSelecting: false, headerLeftSelecting: false });
    }
}
export const escape = new EscapeAction().bind();
class ChooseAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { choosing: payload, entering: true });
    }
}
export const choose = new ChooseAction().bind();
class SelectAction extends CoreAction {
    reduce(store, payload) {
        return Object.assign(Object.assign({}, store), { selectingZone: payload, headerTopSelecting: false, headerLeftSelecting: false });
    }
}
export const select = new SelectAction().bind();
class SelectRowsAction extends CoreAction {
    reduce(store, payload) {
        const { range, numCols } = payload;
        const { start, end } = range;
        const selectingZone = {
            startY: start,
            startX: 1,
            endY: end,
            endX: numCols,
        };
        return Object.assign(Object.assign({}, store), { selectingZone, choosing: { y: start, x: 0 }, headerLeftSelecting: true, headerTopSelecting: false });
    }
}
export const selectRows = new SelectRowsAction().bind();
class SelectColsAction extends CoreAction {
    reduce(store, payload) {
        const { range, numRows } = payload;
        const { start, end } = range;
        const selectingZone = {
            startY: 1,
            startX: start,
            endY: numRows,
            endX: end,
        };
        return Object.assign(Object.assign({}, store), { selectingZone, choosing: { y: 0, x: start }, headerLeftSelecting: false, headerTopSelecting: true });
    }
}
export const selectCols = new SelectColsAction().bind();
class DragAction extends CoreAction {
    reduce(store, payload) {
        const { y, x } = store.choosing;
        const selectingZone = {
            startY: y,
            startX: x,
            endY: payload.y,
            endX: payload.x,
        };
        return Object.assign(Object.assign({}, store), { selectingZone });
    }
}
export const drag = new DragAction().bind();
class SearchAction extends CoreAction {
    reduce(store, payload) {
        let { matchingCells, matchingCellIndex } = store;
        matchingCellIndex += payload;
        if (matchingCellIndex >= matchingCells.length) {
            matchingCellIndex = 0;
        }
        else if (matchingCellIndex < 0) {
            matchingCellIndex = matchingCells.length - 1;
        }
        return Object.assign(Object.assign({}, store), { matchingCells, matchingCellIndex });
    }
}
export const search = new SearchAction().bind();
class WriteAction extends CoreAction {
    reduce(store, payload) {
        const { choosing, selectingZone, table } = store;
        const newTable = table.write({
            point: choosing,
            value: payload,
            reflection: {
                selectingZone,
                choosing,
            },
        });
        return Object.assign(Object.assign(Object.assign({}, store), initSearchStatement(newTable, store)), { table: newTable, copyingZone: { startY: -1, startX: -1, endY: -1, endX: -1 } });
    }
}
export const write = new WriteAction().bind();
class ClearAction extends CoreAction {
    reduce(store, payload) {
        const { choosing, selectingZone, table } = store;
        let selectingArea = zoneToArea(selectingZone);
        if (selectingArea.top === -1) {
            const { y, x } = choosing;
            selectingArea = { top: y, left: x, bottom: y, right: x };
        }
        const { top, left, bottom, right } = selectingArea;
        const diff = {};
        for (let y = top; y <= bottom; y++) {
            for (let x = left; x <= right; x++) {
                diff[p2a({ y, x })] = { value: null };
            }
        }
        const newTable = table.update({
            diff,
            partial: true,
            reflection: {
                selectingZone,
                choosing,
            },
        });
        return Object.assign(Object.assign(Object.assign({}, store), initSearchStatement(newTable, store)), { table: newTable });
    }
}
export const clear = new ClearAction().bind();
class UndoAction extends CoreAction {
    reduce(store, payload) {
        const { table } = store;
        const { history, newTable } = table.undo();
        if (history == null) {
            return store;
        }
        const { reflection, operation } = history;
        return Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, store), restrictPoints(store, table)), reflection), initSearchStatement(newTable, store)), { table: newTable });
    }
}
export const undo = new UndoAction().bind();
class RedoAction extends CoreAction {
    reduce(store, payload) {
        const { table } = store;
        const { history, newTable } = table.redo();
        if (history == null) {
            return store;
        }
        const { reflection, operation } = history;
        return Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, store), reflection), restrictPoints(store, table)), initSearchStatement(newTable, store)), { table: newTable });
    }
}
export const redo = new RedoAction().bind();
class ArrowAction extends CoreAction {
    reduce(store, payload) {
        var _a, _b, _c, _d;
        const { shiftKey, deltaY, deltaX, numRows, numCols } = payload;
        let { choosing, selectingZone, table, gridOuterRef } = store;
        const { y, x } = choosing;
        if (shiftKey) {
            let [dragEndY, dragEndX] = [
                selectingZone.endY === -1 ? y : selectingZone.endY,
                selectingZone.endX === -1 ? x : selectingZone.endX,
            ];
            const [nextY, nextX] = [dragEndY + deltaY, dragEndX + deltaX];
            if (nextY < 1 || numRows < nextY || nextX < 1 || numCols < nextX) {
                return store;
            }
            selectingZone =
                y === nextY && x === nextX
                    ? { startY: -1, startX: -1, endY: -1, endX: -1 }
                    : { startY: y, startX: x, endY: nextY, endX: nextX };
            return Object.assign(Object.assign({}, store), { selectingZone });
        }
        const [nextY, nextX] = [y + deltaY, x + deltaX];
        if (nextY < 1 || numRows < nextY || nextX < 1 || numCols < nextX) {
            return store;
        }
        let { y: editorTop, x: editorLeft, height, width } = store.editorRect;
        if (deltaY > 0) {
            for (let i = y; i < nextY; i++) {
                editorTop += ((_a = table.getByPoint({ y: i, x: 0 })) === null || _a === void 0 ? void 0 : _a.height) || DEFAULT_HEIGHT;
            }
        }
        else if (deltaY < 0) {
            for (let i = y - 1; i >= nextY; i--) {
                editorTop -= ((_b = table.getByPoint({ y: i, x: 0 })) === null || _b === void 0 ? void 0 : _b.height) || DEFAULT_HEIGHT;
            }
        }
        if (deltaX > 0) {
            for (let i = x; i < nextX; i++) {
                editorLeft += ((_c = table.getByPoint({ y: 0, x: i })) === null || _c === void 0 ? void 0 : _c.width) || DEFAULT_WIDTH;
            }
        }
        else if (deltaX < 0) {
            for (let i = x - 1; i >= nextX; i--) {
                editorLeft -= ((_d = table.getByPoint({ y: 0, x: i })) === null || _d === void 0 ? void 0 : _d.width) || DEFAULT_WIDTH;
            }
        }
        const cell = table.getByPoint({ y: nextY, x: nextX });
        height = (cell === null || cell === void 0 ? void 0 : cell.height) || DEFAULT_HEIGHT;
        width = (cell === null || cell === void 0 ? void 0 : cell.width) || DEFAULT_WIDTH;
        smartScroll(table, gridOuterRef.current, { y: nextY, x: nextX });
        return Object.assign(Object.assign({}, store), { selectingZone: { startY: -1, startX: -1, endY: -1, endX: -1 }, choosing: { y: nextY, x: nextX }, editorRect: { y: editorTop, x: editorLeft, height, width } });
    }
}
export const arrow = new ArrowAction().bind();
class WalkAction extends CoreAction {
    reduce(store, payload) {
        var _a, _b, _c, _d;
        let { deltaY, deltaX, numRows, numCols } = payload;
        let { choosing, selectingZone, table, gridOuterRef } = store;
        let { y: editorTop, x: editorLeft, height, width } = store.editorRect;
        const { y, x } = choosing;
        const selectingArea = zoneToArea(selectingZone);
        const { top, left, bottom, right } = selectingArea;
        let [nextY, nextX] = [y + deltaY, x + deltaX];
        if (nextY < top && top !== -1) {
            deltaY = bottom - nextY;
            nextY = bottom;
            if (nextX > left) {
                nextX--;
                deltaX--;
            }
            else {
                deltaX = right - nextX;
                nextX = right;
            }
        }
        if (nextY > bottom && bottom !== -1) {
            deltaY = top - nextY;
            nextY = top;
            if (nextX < right) {
                nextX++;
                deltaX++;
            }
            else {
                deltaX = left - nextX;
                nextX = left;
            }
        }
        if (nextX < left && left !== -1) {
            deltaX = right - nextX;
            nextX = right;
            if (nextY > top) {
                nextY--;
                deltaY--;
            }
            else {
                deltaY = bottom - nextY;
                nextY = bottom;
            }
        }
        if (nextX > right && right !== -1) {
            deltaX = left - nextX;
            nextX = left;
            if (nextY < bottom) {
                nextY++;
                deltaY++;
            }
            else {
                deltaY = top - nextY;
                nextY = top;
            }
        }
        if (nextY < 1 || numRows < nextY || nextX < 1 || numCols < nextX) {
            return store;
        }
        if (deltaY > 0) {
            for (let i = y; i < nextY; i++) {
                editorTop += ((_a = table.getByPoint({ y: i, x: 0 })) === null || _a === void 0 ? void 0 : _a.height) || DEFAULT_HEIGHT;
            }
        }
        else if (deltaY < 0) {
            for (let i = y - 1; i >= nextY; i--) {
                editorTop -= ((_b = table.getByPoint({ y: i, x: 0 })) === null || _b === void 0 ? void 0 : _b.height) || DEFAULT_HEIGHT;
            }
        }
        if (deltaX > 0) {
            for (let i = x; i < nextX; i++) {
                editorLeft += ((_c = table.getByPoint({ y: 0, x: i })) === null || _c === void 0 ? void 0 : _c.width) || DEFAULT_WIDTH;
            }
        }
        else if (deltaX < 0) {
            for (let i = x - 1; i >= nextX; i--) {
                editorLeft -= ((_d = table.getByPoint({ y: 0, x: i })) === null || _d === void 0 ? void 0 : _d.width) || DEFAULT_WIDTH;
            }
        }
        const cell = table.getByPoint({ y: nextY, x: nextX });
        height = (cell === null || cell === void 0 ? void 0 : cell.height) || DEFAULT_HEIGHT;
        width = (cell === null || cell === void 0 ? void 0 : cell.width) || DEFAULT_WIDTH;
        smartScroll(table, gridOuterRef.current, { y: nextY, x: nextX });
        return Object.assign(Object.assign({}, store), { choosing: { y: nextY, x: nextX }, editorRect: { y: editorTop, x: editorLeft, height, width } });
    }
}
export const walk = new WalkAction().bind();
//# sourceMappingURL=actions.js.map